local M = {}
local config = require("odis.config")
local window = require("odis.window")
local utils = require("odis.utils")

function M.display_documentation(doc_lines, display_mode, source_type)
    if not doc_lines or #doc_lines == 0 then return false end

    if display_mode == config.modes.float then
        return window.display_float(doc_lines, {
            title = string.format(" %s Documentation ", source_type),
            filetype = source_type:lower(),
        })
    end

    local buf = vim.api.nvim_create_buf(false, true)
    vim.api.nvim_buf_set_lines(buf, 0, -1, false, doc_lines)
    
    for opt, value in pairs(window.buf_opts) do
        vim.api.nvim_buf_set_option(buf, opt, value)
    end
    vim.api.nvim_buf_set_option(buf, "filetype", source_type:lower())
    utils.setup_treesitter(buf, source_type)

    return window.create_documentation_window(buf, display_mode, source_type or "generic")
end

function M.display_vim_help(word, display_mode)
    if display_mode == config.modes.float then
        local ok, _ = pcall(vim.cmd, "help " .. word)
        if not ok then return false end

        local help_buf = vim.api.nvim_get_current_buf()
        local cursor_pos = vim.api.nvim_win_get_cursor(0)
        local lines = vim.api.nvim_buf_get_lines(help_buf, 0, -1, false)
        vim.cmd("bdelete " .. help_buf)

        local win, buf = window.display_float(lines, {
            title = " Vim Help: " .. word .. " ",
            filetype = "help",
        })
        
        if win then
            vim.api.nvim_win_set_cursor(win, cursor_pos)
            vim.api.nvim_win_call(win, function()
                vim.cmd("normal! gg")
                vim.cmd("/" .. vim.fn.escape(word, "/\\"))
                vim.cmd("normal! zz")
            end)
        end

        return win, buf
    end

    local buf = vim.api.nvim_create_buf(false, true)
    local ok, _ = pcall(vim.cmd, "help " .. word)
    if not ok then return false end

    local help_buf = vim.api.nvim_get_current_buf()
    local cursor_pos = vim.api.nvim_win_get_cursor(0)
    local lines = vim.api.nvim_buf_get_lines(help_buf, 0, -1, false)
    vim.cmd("bdelete " .. help_buf)

    vim.api.nvim_buf_set_lines(buf, 0, -1, false, lines)
    for opt, value in pairs(window.buf_opts) do
        vim.api.nvim_buf_set_option(buf, opt, value)
    end

    vim.api.nvim_buf_set_option(buf, "filetype", "help")
    vim.api.nvim_buf_set_option(buf, "iskeyword", "!-~,^*,^|,^\"")
    vim.api.nvim_buf_set_option(buf, "tabstop", 8)

    local win = window.create_documentation_window(buf, display_mode, "vim")
    if win then
        vim.api.nvim_win_set_option(win, "conceallevel", 2)
        vim.api.nvim_win_set_option(win, "concealcursor", "nc")
        vim.api.nvim_win_set_option(win, "spell", false)
        
        vim.api.nvim_win_set_cursor(win, cursor_pos)
        vim.api.nvim_win_call(win, function()
            vim.cmd("normal! gg")
            vim.cmd("/" .. vim.fn.escape(word, "/\\"))
            vim.cmd("normal! zz")
        end)
    end

    return win
end

function M.display_man_page(word, display_mode)
    if display_mode == config.modes.float then
        vim.cmd("Man " .. word)
        local man_buf = vim.api.nvim_get_current_buf()
        local lines = vim.api.nvim_buf_get_lines(man_buf, 0, -1, false)
        vim.cmd("bdelete")

        return window.display_float(lines, {
            title = " Man: " .. word .. " ",
            filetype = "man",
        })
    end

    local buf = vim.api.nvim_create_buf(false, true)
    vim.cmd("Man " .. word)
    local man_buf = vim.api.nvim_get_current_buf()
    local lines = vim.api.nvim_buf_get_lines(man_buf, 0, -1, false)
    vim.cmd("bdelete")

    vim.api.nvim_buf_set_lines(buf, 0, -1, false, lines)
    for opt, value in pairs(window.buf_opts) do
        vim.api.nvim_buf_set_option(buf, opt, value)
    end
    vim.api.nvim_buf_set_option(buf, "filetype", "man")

    return window.create_documentation_window(buf, display_mode, "man")
end

function M.get_documentation(word, callback)
    local sources = {}
    local opts = config.options.sources
    local pending = 0
    local completed = false

    -- Track enabled sources
    local enabled_count = 0
    if opts.vim.enabled then enabled_count = enabled_count + 1 end
    if opts.man.enabled then enabled_count = enabled_count + 1 end
    if opts.lsp.enabled then enabled_count = enabled_count + 1 end

    -- Early return if no sources are enabled
    if enabled_count == 0 then
        vim.notify("No documentation sources are enabled", vim.log.levels.WARN)
        return
    end

    local function process_sources()
        if completed and pending == 0 then
            if #sources > 0 then
                local priority_map = {}
                for i, p in ipairs(config.options.priority) do
                    priority_map[p:lower()] = i
                end
                table.sort(sources, function(a, b)
                    return (priority_map[a.name:lower()] or 99) < (priority_map[b.name:lower()] or 99)
                end)
                callback(sources)
            else
                vim.notify("No documentation available for '" .. word .. "'", vim.log.levels.INFO)
            end
        end
    end

    -- Vim help check
    if opts.vim.enabled then
        pending = pending + 1
        vim.schedule(function()
            local temp_buf = vim.api.nvim_create_buf(false, true)
            local ok = pcall(function()
                vim.api.nvim_set_current_buf(temp_buf)
                vim.cmd("help " .. word)
                local help_buf = vim.api.nvim_get_current_buf()
                if help_buf ~= temp_buf then
                    local lines = vim.api.nvim_buf_get_lines(help_buf, 0, -1, false)
                    if #lines > 0 then
                        table.insert(sources, { name = "Vim", doc = lines })
                    end
                    vim.cmd(help_buf .. "bdelete!")
                end
            end)
            vim.cmd(temp_buf .. "bdelete!")
            pending = pending - 1
            process_sources()
        end)
    end

    -- Man page check
    if opts.man.enabled then
        pending = pending + 1
        vim.fn.jobstart({"man", "-P", "cat", word}, {
            stdout_buffered = true,
            on_stdout = function(_, data)
                if data and #data > 1 then
                    table.insert(sources, { name = "Man", doc = data })
                end
            end,
            on_exit = function(_, code)
                vim.schedule(function()
                    pending = pending - 1
                    process_sources()
                end)
            end
        })
    end

    -- LSP documentation
    if opts.lsp.enabled then
        local clients = vim.lsp.get_active_clients({ bufnr = 0 })
        for _, client in ipairs(clients) do
            if client.server_capabilities.hoverProvider then
                pending = pending + 1
                local params = vim.lsp.util.make_position_params()
                vim.lsp.buf_request(0, "textDocument/hover", params, function(_, result)
                    if result and result.contents then
                        local doc_lines = vim.lsp.util.convert_input_to_markdown_lines(result.contents)
                        if doc_lines and #doc_lines > 0 then
                            table.insert(sources, { name = "LSP", doc = doc_lines })
                        end
                    end
                    pending = pending - 1
                    process_sources()
                end)
                break
            end
        end
    end

    completed = true
    process_sources()
end

return M
