local M = {}
local config = require("odis.config")
local utils = require("odis.utils")

M.state = {
    windows = {},  -- Stack of {win = win_id, buf = buf_id, mode = display_mode}
}

M.buf_opts = {
    modified = false,
    modifiable = false,
    buftype = "nofile",
    bufhidden = "wipe",
    swapfile = false,
    buflisted = false,
}

M.win_opts = {
    wrap = true,
    number = false,
    relativenumber = false,
    cursorline = false,
    signcolumn = "no",
}

function M.track_window(win, buf, display_mode)
    if win then
        table.insert(M.state.windows, {
            win = win,
            buf = buf,
            mode = display_mode
        })
    end
end

function M.close_window()
    local last = M.state.windows[#M.state.windows]
    if not last or not vim.api.nvim_win_is_valid(last.win) then
        table.remove(M.state.windows)
        return
    end

    if last.mode == config.modes.buffer then
        vim.cmd("buffer #")
    else
        vim.api.nvim_win_close(last.win, true)
    end

    table.remove(M.state.windows)
end

function M.calculate_float_position(width, height, cfg)
    local editor_height = vim.api.nvim_get_option("lines")
    local editor_width = vim.api.nvim_get_option("columns")
    local pos = {}

    local border_space = cfg.border ~= "none" and 2 or 0
    width = width + border_space
    height = height + border_space

    if cfg.anchor == "cursor" then
        pos = {
            relative = "cursor",
            row = cfg.offset.row,
            col = cfg.offset.col,
        }
    else
        pos = {
            relative = "editor",
            row = 0,
            col = 0,
        }

        if cfg.anchor == "bottom_right" then
            pos.row = editor_height - height - 2 + cfg.offset.row
            pos.col = editor_width - width - 2 + cfg.offset.col
        elseif cfg.anchor == "bottom_left" then
            pos.row = editor_height - height - 2 + cfg.offset.row
            pos.col = 1 + cfg.offset.col
        elseif cfg.anchor == "top_right" then
            pos.row = 1 + cfg.offset.row
            pos.col = editor_width - width - 2 + cfg.offset.col
        elseif cfg.anchor == "top_left" then
            pos.row = 1 + cfg.offset.row
            pos.col = 1 + cfg.offset.col
        end

        pos.row = math.max(0, math.min(pos.row, editor_height - height - 2))
        pos.col = math.max(0, math.min(pos.col, editor_width - width - 2))
    end

    return pos
end

function M.display_float(content, opts)
    local float_config = config.options.display.float
    local buf = vim.api.nvim_create_buf(false, true)

    content = type(content) == "string" and vim.split(content, "\n") or content
    vim.api.nvim_buf_set_lines(buf, 0, -1, false, content)

    for k, v in pairs(M.buf_opts) do
        vim.api.nvim_buf_set_option(buf, k, v)
    end

    local max_content_width = 0
    for _, line in ipairs(content) do
        max_content_width = math.max(max_content_width, vim.fn.strdisplaywidth(line))
    end

    local width = math.min(max_content_width + 2, float_config.maxwidth)
    local height = math.min(#content, float_config.maxheight)
    local pos = M.calculate_float_position(width, height, float_config)

    local win = vim.api.nvim_open_win(buf, float_config.auto_focus, {
        relative = pos.relative,
        width = width,
        height = height,
        row = pos.row,
        col = pos.col,
        style = float_config.style,
        border = float_config.border,
        zindex = 50,
        title = (float_config.title and opts and opts.title) and opts.title or nil,
        title_pos = "center",
        focusable = true,
    })

    for k, v in pairs(M.win_opts) do
        pcall(vim.api.nvim_win_set_option, win, k, v)
    end

    vim.api.nvim_win_set_option(win, "winhighlight", "NormalFloat:Normal")

    if opts and opts.filetype then
        vim.api.nvim_buf_set_option(buf, "filetype", opts.filetype)
        utils.setup_treesitter(buf, opts.filetype)
    end

    M.track_window(win, buf, config.modes.float)
    return win, buf
end

function M.setup_window_options(win, source_type, buf)
    for k, v in pairs(M.win_opts) do
        pcall(vim.api.nvim_win_set_option, win, k, v)
    end

    if config.options.mappings.close then
        vim.keymap.set("n", config.options.mappings.close, M.close_window, { buffer = buf, silent = true })
    end

    M.track_window(win, buf, source_type)
end

function M.get_split_position(display_mode, window_config)
    local position = window_config.position
    if position:find("|") then
        local pos1, pos2 = position:match("([^|]+)|([^|]+)")
        if display_mode == config.modes.vsplit then
            return pos1 == "left" or pos1 == "right" and pos1 or pos2
        else
            return pos1 == "top" or pos1 == "bottom" and pos1 or pos2
        end
    end
    return position
end

function M.create_documentation_window(buf, display_mode, source_type)
    local window_config = config.options.display.window
    local cur_win = vim.api.nvim_get_current_win()
    local win

    if display_mode == config.modes.buffer then
        M.state.origin_buf = vim.api.nvim_get_current_buf()
        vim.api.nvim_win_set_buf(cur_win, buf)
        win = cur_win
    elseif display_mode == config.modes.tab then
        vim.cmd("tabnew")
        win = vim.api.nvim_get_current_win()
        vim.api.nvim_win_set_buf(win, buf)
    else
        local dimensions = {
            width = math.floor(vim.api.nvim_win_get_width(cur_win) * window_config.width),
            height = math.floor(vim.api.nvim_win_get_height(cur_win) * window_config.height)
        }

        if window_config.floating and window_config.border and window_config.border ~= "none" then
            win = vim.api.nvim_open_win(buf, true, {
                relative = 'editor',
                width = dimensions.width,
                height = dimensions.height,
                border = window_config.border,
                style = 'minimal',
                title = " Documentation ",
                title_pos = "center",
            })
        else
            local position = M.get_split_position(display_mode, window_config)
            local split_cmd = display_mode == config.modes.vsplit and
                (position == "right" and "rightbelow vsplit" or "leftabove vsplit") or
                (position == "bottom" and "rightbelow split" or "leftabove split")
            
            vim.cmd(split_cmd)
            win = vim.api.nvim_get_current_win()
            vim.api.nvim_win_set_buf(win, buf)
            
            local resize_cmd = display_mode == config.modes.vsplit and 
                "vertical resize " .. dimensions.width or
                "resize " .. dimensions.height
            vim.cmd(resize_cmd)

            for k, v in pairs(M.win_opts) do
                pcall(vim.api.nvim_win_set_option, win, k, v)
            end
        end
    end

    M.setup_window_options(win, display_mode, buf)
    return win
end

return M
